@@ -3,7 +3,7 @@
 ;; Copyright (C) 2014  Free Software Foundation, Inc.
 
 ;; Author: Eric Abrahamsen <eric@ericabrahamsen.net>
-;; Keywords: 
+;; Keywords:
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 ;;; Commentary:
 
-;; 
+;;
 
 ;;; Code:
 
@@ -93,6 +93,17 @@ Basically behave as if all attachments have \":gnus-attachments t\"."
   :group 'gnorb-gnus
   :type 'string)
 
+(defcustom gnorb-gnus-copy-message-text nil
+  "When capturing or triggering a TODO from a Gnus message,
+should the text of the message be saved?
+
+If t, the body text of the message is pushed onto the kill ring.
+If a char value, the text is saved into the corresponding
+register."
+  :group 'gnorb-gnus
+  :type '(choice boolean
+		 character))
+
 (defcustom gnorb-gnus-hint-relevant-article t
   "When opening a gnus message, should gnorb let you know if the
   message is relevant to an existing TODO?"
@@ -246,7 +257,27 @@ save them into `gnorb-tmp-dir'."
      '("file" "files" "attach"))
     (setq gnorb-gnus-capture-attachments nil)))
 
+(defun gnorb-gnus-capture-save-text ()
+  (when (and gnorb-gnus-copy-message-text
+	     (with-current-buffer
+		 (org-capture-get :original-buffer)
+	       (memq major-mode '(gnus-summary-mode gnus-article-mode))))
+    (save-window-excursion
+      (set-buffer (org-capture-get :original-buffer))
+      (gnus-with-article-buffer
+	(article-goto-body)
+	(if (numberp gnorb-gnus-copy-message-text)
+	    (progn
+	      (copy-to-register
+	       gnorb-gnus-copy-message-text
+	       (point) (point-max))
+	      (message "Message text copied to register %c"
+		       gnorb-gnus-copy-message-text))
+	  (kill-new (buffer-substring (point) (point-max)))
+	  (message "Message text copied to kill ring"))))))
+
 (add-hook 'org-capture-mode-hook 'gnorb-gnus-capture-attach)
+(add-hook 'org-capture-mode-hook 'gnorb-gnus-capture-save-text)
 
 (defvar org-note-abort)
 
@@ -523,7 +554,8 @@ you'll stay in the Gnus summary buffer."
   (setq gnorb-window-conf (current-window-configuration))
   (move-marker gnorb-return-marker (point))
   (setq gnorb-gnus-message-info nil)
-  (let* ((articles (gnus-summary-work-articles arg))
+  (let* ((buf (current-buffer))
+	 (articles (gnus-summary-work-articles arg))
 	 (art-no (gnus-summary-article-number))
 	 (headers (gnus-data-header
 		   (gnus-data-find art-no)))
@@ -535,6 +567,10 @@ you'll stay in the Gnus summary buffer."
 	 (group (gnorb-get-real-group-name
 		 gnus-newsgroup-name
 		 art-no))
+	 (text (gnus-with-article-buffer
+		 (article-goto-body)
+		 (buffer-substring-no-properties
+		  (point) (point-max))))
 	 (link (call-interactively 'org-store-link))
 	 (org-refile-targets gnorb-gnus-trigger-refile-targets)
 	 (ref-msg-ids (concat (mail-header-references headers) " "
@@ -552,57 +588,71 @@ you'll stay in the Gnus summary buffer."
 		     :group ,group))
     (gnorb-gnus-collect-all-attachments nil t)
     (condition-case err
-     (if id
-	 (progn
-	   (delete-other-windows)
-	   (gnorb-trigger-todo-action nil id))
-       ;; Flush out zombies (dead associations).
-       (setq related-headings
-	     (cl-remove-if
-	      (lambda (h)
-		(when (null (org-id-find-id-file h))
-		  (when (y-or-n-p
-			 (format
-			  "ID %s no longer exists, disassociate message?"
-			  h))
-		    (gnorb-delete-association msg-id h))))
-	      related-headings))
-       ;; See if one of the related headings is chosen.
-       (unless (catch 'target
-		 (dolist (h related-headings nil)
-		   (when (yes-or-no-p
-			  (format "Trigger action on %s"
-				  (gnorb-pretty-outline h)))
-		     (throw 'target (setq targ h)))))
-	 ;; If not, use the refile interface to choose one.
-	 (setq targ (org-refile-get-location
-		     "Trigger heading" nil t))
-	 (setq targ
-	       (save-window-excursion
-		 (find-file (nth 1 targ))
-		 (goto-char (nth 3 targ))
-		 (org-id-get-create))))
-       ;; Either bulk associate multiple messages...
-       (if (> (length articles) 1)
-	   (progn
-	     (dolist (a articles)
-	       (gnorb-registry-make-entry
-		(mail-header-id
-		 (gnus-data-header
-		  (gnus-data-find a)))
-		from subject targ group)
-	       (gnus-summary-remove-process-mark a))
-	     (message "Associated %d messages with %s"
-		      (length articles) (gnorb-pretty-outline targ)))
-	 ;; ...or just trigger the one.
-	 (delete-other-windows)
-	 (gnorb-trigger-todo-action nil targ)))
-     (error
-      ;; If these are left populated after an error, it plays hell
-      ;; with future trigger processes.
-      (setq gnorb-gnus-message-info nil)
-      (setq gnorb-gnus-capture-attachments nil)
-      (signal (car err) (cdr err))))))
+	(if id
+	    (progn
+	      (delete-other-windows)
+	      (gnorb-trigger-todo-action nil id))
+	  ;; Flush out zombies (dead associations).
+	  (setq related-headings
+		(cl-remove-if
+		 (lambda (h)
+		   (when (null (org-id-find-id-file h))
+		     (when (y-or-n-p
+			    (format
+			     "ID %s no longer exists, disassociate message?"
+			     h))
+		       (gnorb-delete-association msg-id h))))
+		 related-headings))
+	  ;; See if one of the related headings is chosen.
+	  (unless (catch 'target
+		    (dolist (h related-headings nil)
+		      (when (yes-or-no-p
+			     (format "Trigger action on %s"
+				     (gnorb-pretty-outline h)))
+			(throw 'target (setq targ h)))))
+	    ;; If not, use the refile interface to choose one.
+	    (setq targ (org-refile-get-location
+			"Trigger heading" nil t))
+	    (setq targ
+		  (save-window-excursion
+		    (find-file (nth 1 targ))
+		    (goto-char (nth 3 targ))
+		    (org-id-get-create))))
+	  ;; Either bulk associate multiple messages...
+	  (if (> (length articles) 1)
+	      (progn
+		(dolist (a articles)
+		  (gnorb-registry-make-entry
+		   (mail-header-id
+		    (gnus-data-header
+		     (gnus-data-find a)))
+		   from subject targ group)
+		  (gnus-summary-remove-process-mark a))
+		(message "Associated %d messages with %s"
+			 (length articles) (gnorb-pretty-outline targ)))
+	    ;; ...or just trigger the one.
+	    (delete-other-windows)
+	    (gnorb-trigger-todo-action nil targ)
+	    (when gnorb-gnus-copy-message-text
+	      (if (numberp gnorb-gnus-copy-message-text)
+		  (with-temp-buffer
+		    (insert text)
+		    (copy-to-register
+		     gnorb-gnus-copy-message-text
+		     (point-min) (point-max))
+		    (message "Message text copied to register %c"
+			     gnorb-gnus-copy-message-text))
+		(kill-new text)
+		(message "Message text copied to kill ring"))))
+	  (with-current-buffer buf
+	    (dolist (a articles)
+	      (gnus-summary-update-article a))))
+      (error
+       ;; If these are left populated after an error, it plays hell
+       ;; with future trigger processes.
+       (setq gnorb-gnus-message-info nil)
+       (setq gnorb-gnus-capture-attachments nil)
+       (signal (car err) (cdr err))))))
 
 ;;;###autoload
 (defun gnorb-gnus-quick-reply ()
@@ -669,40 +719,29 @@ server. There must be an active \"nngnorb\" server for this to
 work."
   (interactive)
   (require 'nnir)
-  (let ((nnir-address
-	 (or (gnus-method-to-server '(nngnorb))
-	     (user-error
-	      "Please add a \"nngnorb\" backend to your gnus installation.")))
-	name method spec)
-    (when (version= "5.13" gnus-version-number)
-      (with-no-warnings		  ; All these variables are available.
-	(setq nnir-current-query nil
-	      nnir-current-server nil
-	      nnir-current-group-marked nil
-	      nnir-artlist nil)))
-    ;; In 24.4, the group name is mostly decorative, but in 24.3, the
-    ;; actual query is held there.
-    (setq name (if (version= "5.13" gnus-version-number)
-		   (concat "nnir:" (prin1-to-string `((query ,str))))
-		 (if persist
-		     (read-string
-		      (format "Name for group (default %s): " head-text)
-		      nil head-text t)
-		   (concat "gnorb-" str))))
-    (setq method (if (version= "5.13" gnus-version-number)
-		     (list 'nnir nnir-address)
-		   (list 'nnir "Gnorb")))
-    (setq spec
-	  (list
-	   (cons 'nnir-specs (list (cons 'nnir-query-spec `((query . ,str)))
-				   (cons 'nnir-group-spec `((,nnir-address nil)))))
-	   (cons 'nnir-artlist nil)))
+  (let* ((nnir-address
+	  (or (catch 'found
+		(dolist (s gnus-server-method-cache)
+		  (when (eq 'nngnorb (cadr s))
+		    (throw 'found (car s)))))
+	      (user-error
+	       "Please add a \"nngnorb\" backend to your gnus installation.")))
+	 (name (if persist
+		   (read-string
+		    (format "Name for group (default %s): " head-text)
+		    nil nil head-text)
+		 (concat "gnorb-" str)))
+	 (method (list 'nnir nnir-address))
+	 (spec (list
+		(cons 'nnir-specs (list (cons 'nnir-query-spec `((query . ,str)))
+					(cons 'nnir-group-spec `((,nnir-address ,(list name))))))
+		(cons 'nnir-artlist nil))))
     (if persist
 	(progn
 	  (switch-to-buffer gnus-group-buffer)
 	  (gnus-group-make-group name method nil spec)
 	  (gnus-group-select-group))
-     (gnus-group-read-ephemeral-group name method nil ret nil nil spec))))
+      (gnus-group-read-ephemeral-group name method nil ret nil nil spec))))
 
 (defun gnorb-gnus-summary-mode-hook ()
   "Check if we've entered a Gnorb-generated group, and activate
@@ -756,15 +795,20 @@ option `gnorb-gnus-hint-relevant-article' is non-nil."
 (add-hook 'gnus-select-article-hook 'gnorb-gnus-hint-relevant-message)
 
 (defun gnorb-gnus-insert-format-letter-maybe (header)
-  (if (not (memq (car (gnus-find-method-for-group
-		       gnus-newsgroup-name))
-		 '(nnvirtual nnir)))
-      (cond ((gnus-registry-get-id-key
-	      (mail-header-message-id header) 'gnorb-ids)
-	     gnorb-gnus-summary-tracked-mark)
-	    ((gnorb-find-tracked-headings header)
-	     gnorb-gnus-summary-mark)
-	    (t " "))
+  (if (not (or (gnus-ephemeral-group-p gnus-newsgroup-name)
+	       (gnus-virtual-group-p gnus-newsgroup-name)))
+      (let* ((id (mail-header-message-id header))
+	     ;; Use lower-level accessor to avoid creating an entry
+	     ;; where there wasn't one.  This function doesn't respect
+	     ;; ignored registry groups.
+	     (entry (nth 1 (assoc id (registry-lookup
+				      gnus-registry-db
+				      (list id))))))
+	(cond ((cdr-safe (assq 'gnorb-ids entry))
+	       gnorb-gnus-summary-tracked-mark)
+	      ((gnorb-find-tracked-headings header)
+	       gnorb-gnus-summary-mark)
+	      (t " ")))
     " "))
 
 (fset (intern (concat "gnus-user-format-function-"
