@@ -24,7 +24,9 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl-lib))
+(eval-when-compile
+  (require 'cl-lib)
+  (require 'nnheader))
 (require 'pcase)
 (require 'org)
 (require 'org-agenda)
@@ -76,6 +78,13 @@ with `gnorb-window-conf'.")
   :group 'gnorb
   :type 'string)
 
+(defun gnorb-version ()
+  "Return the version of currently-installed Gnorb.
+
+Only works for Gnorb installed via the package manager."
+  (interactive)
+  (pkg-info-package-version "gnorb"))
+
 ;;; this is just ghastly, but the value of this var is single regexp
 ;;; group containing various header names, and we want our value
 ;;; inside that group.
@@ -481,14 +490,14 @@ and `gnus'."
 (defun gnorb-msg-id-to-link (msg-id)
   "Given a message id, try to create a full org link to the
 message."
-  (let ((server-group (gnorb-msg-id-to-group msg-id)))
+  (let ((server-group (car (gnorb-msg-id-request-head msg-id))))
     (when server-group
       (org-link-escape
        (concat server-group "#"
 	       (gnorb-unbracket-message-id msg-id))))))
 
-(defun gnorb-msg-id-to-group (msg-id)
-  "Given a message id, try to find the group it's in.
+(defun gnorb-msg-id-request-head (msg-id)
+  "Given a message id, try to find its group and article number.
 
 So far we're checking the registry, then the groups in
 `gnorb-gnus-sent-groups'. Use search engines? Other clever
@@ -497,24 +506,22 @@ methods?"
     (setq msg-id (gnorb-bracket-message-id msg-id))
     (catch 'found
       (when gnorb-tracking-enabled
-	(setq candidates (gnus-registry-get-id-key msg-id 'group))
-	(if (= 1 (length candidates))
-	    (throw 'found (car candidates))
-	  (setq candidates (append candidates gnorb-gnus-sent-groups))
-	  (while (setq server-group (pop candidates))
-	    (when (and (stringp server-group)
-		       (string-match-p "+" server-group)
-		       (not
-			(string-match-p
-			 "\\(nnir\\|nnvirtual\\|UNKNOWN\\)"
-			 server-group)))
-	      (setq check
-		    (ignore-errors
-		      (gnus-request-head msg-id server-group)))
-	      (when check
-		(gnus-registry-set-id-key msg-id 'group (list server-group))
-		(gnus-registry-set-id-key msg-id 'artno (list (cdr check)))
-		(throw 'found (car check)))))))
+	(setq candidates (append (gnus-registry-get-id-key msg-id 'group)
+				 gnorb-gnus-sent-groups))
+	(while (setq server-group (pop candidates))
+	  (when (and (stringp server-group)
+		     (string-match-p "+" server-group)
+		     (not
+		      (string-match-p
+		       "\\(nnir\\|nnvirtual\\|UNKNOWN\\)"
+		       server-group))
+		     (gnus-activate-group server-group)
+		     (setq check
+			   (ignore-errors
+			     (gnus-request-head msg-id server-group))))
+
+	    (gnus-registry-set-id-key msg-id 'group (list server-group))
+	    (throw 'found (cons server-group (cdr check))))))
       nil)))
 
 (defun gnorb-collect-ids (&optional id)
@@ -595,7 +602,6 @@ registry be in use, and should be called after the call to
   (require 'gnorb-registry)
   (with-eval-after-load 'gnus-registry
     (add-to-list 'gnus-registry-extra-entries-precious 'gnorb-ids)
-    (add-to-list 'gnus-registry-extra-entries-precious 'artno)
     (add-to-list 'gnus-registry-track-extra 'gnorb-ids))
   (add-hook
    'gnus-started-hook
@@ -611,5 +617,48 @@ registry be in use, and should be called after the call to
      (add-hook 'org-capture-prepare-finalize-hook 'gnorb-registry-capture-abort-cleanup)
      (setq gnorb-tracking-enabled t))))
 
+;;;###autoload
+(defun gnorb-install-defaults ()
+  "Set up sane Gnorb customizations and keybindings."
+  (interactive)
+  (global-set-key (kbd "C-c A") 'gnorb-restore-layout)
+  (eval-after-load "gnorb-bbdb"
+    '(progn
+       (define-key bbdb-mode-map (kbd "C-c S") #'gnorb-bbdb-mail-search)
+       (define-key bbdb-mode-map (kbd "C-c l") #'gnorb-bbdb-open-link)
+       (define-key bbdb-mode-map [remap bbdb-mail] #'gnorb-bbdb-mail)
+       (eval-after-load "gnorb-org"
+	 (org-defkey org-mode-map (kbd "C-c C") #'gnorb-org-contact-link))))
+  (eval-after-load "gnorb-org"
+    '(progn
+       (org-defkey org-mode-map (kbd "C-c t") #'gnorb-org-handle-mail)
+       (org-defkey org-mode-map (kbd "C-c v") #'gnorb-org-view)
+       (org-defkey org-mode-map (kbd "C-c E") #'gnorb-org-email-subtree)
+       (setq gnorb-org-agenda-popup-bbdb t)
+       (eval-after-load "org-agenda"
+         '(progn (org-defkey org-agenda-mode-map (kbd "C-c t") #'gnorb-org-handle-mail)
+                 (org-defkey org-agenda-mode-map (kbd "C-c v") #'gnorb-org-view)))))
+  (eval-after-load "gnorb-gnus"
+    '(progn
+       (define-key gnus-summary-mime-map "a" #'gnorb-gnus-article-org-attach)
+       (define-key gnus-summary-mode-map (kbd "C-c t") #'gnorb-gnus-incoming-do-todo)
+       (define-key gnus-summary-mode-map (kbd "C-c v") #'gnorb-gnus-view)
+       (setq gnorb-gnus-capture-always-attach t)
+       (push '("attach to org heading" . gnorb-gnus-mime-org-attach)
+             gnus-mime-action-alist)
+       ;; The only way to add mime button command keys is by redefining
+       ;; gnus-mime-button-map, possibly not ideal. Ideal would be a
+       ;; setter function in gnus itself.
+       (push '(gnorb-gnus-mime-org-attach "a" "Attach to Org heading")
+             gnus-mime-button-commands)
+       (setq gnus-mime-button-map
+             (let ((map (make-sparse-keymap)))
+               (dolist (c gnus-mime-button-commands)
+                 (define-key map (cadr c) (car c)))
+               map))))
+  (eval-after-load "message"
+    '(progn
+       (define-key message-mode-map (kbd "C-c t") #'gnorb-gnus-outgoing-do-todo))))
+
 (provide 'gnorb-utils)
 ;;; gnorb-utils.el ends here
